#include "abstracttablewidget.h"
#include <QVBoxLayout>
#include <QAction>
#include <QtSql>
#include <QDebug>
#include <QFileDialog>
#include <QDesktopServices>

AbstractTableWidget::AbstractTableWidget(QWidget *parent) :
        QWidget(parent)
{

    mToolBar = new QToolBar;
    mTreeView= new QTreeView;
    mLineEdit=  new QLineEdit;
    mComboField = new QComboBox;
    mFilterModel = 0;
    mQueryModel = 0;
    mTreeView->setRootIsDecorated(false);
    mTreeView->setAlternatingRowColors(true);
    mTreeView->setSelectionMode(QAbstractItemView::ExtendedSelection);

    QVBoxLayout * layout = new QVBoxLayout;
    QAction * addAction =new QAction(QIcon(":/icons/small/add.png"),"add",this);
    QAction * editAction =new QAction(QIcon(":/icons/small/edit.png"),"edit",this);
    QAction * delAction=new QAction(QIcon(":/icons/small/delete.png"),"delete",this);
    QAction * refreshAction=new QAction(QIcon(":/icons/small/refresh.png"),"refresh",this);
    QAction * exportCsvAction=new QAction(QIcon(":/icons/small/csv.png"),"export csv",this);


    mToolBar->addAction(addAction);
    mToolBar->addAction(editAction);
    mToolBar->addAction(delAction);
    mToolBar->addAction(refreshAction);
    mToolBar->addSeparator();
    mToolBar->addAction(exportCsvAction);


    mToolBar->addWidget(mLineEdit);
    mToolBar->addWidget(mComboField);


    layout->addWidget(mToolBar);
    layout->addWidget(mTreeView);

    setLayout(layout);

    connect(addAction,SIGNAL(triggered()),this,SLOT(addRecord()));
    connect(editAction,SIGNAL(triggered()),this,SLOT(editRecord()));
    connect(delAction,SIGNAL(triggered()),this,SLOT(deleteRecord()));
    connect(refreshAction,SIGNAL(triggered()),this,SLOT(refresh()));
    connect(exportCsvAction,SIGNAL(triggered()),this,SLOT(exportCsv()));
    connect(mTreeView,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(itemSelected(QModelIndex)));
    connect(mLineEdit,SIGNAL(textChanged(QString)),this,SLOT(filterChanged(QString)));

}
void AbstractTableWidget::setModel(QSqlQueryModel * model)
{
    mQueryModel=model;
    mFilterModel = new QSortFilterProxyModel;
    mFilterModel->setSourceModel(mQueryModel);
    mTreeView->setModel(mFilterModel);

    QSqlRecord record = mQueryModel->record();
    for ( int i=0; i<record.count(); ++i)
        mComboField->addItem(record.field(i).name());

}

void AbstractTableWidget::refresh()
{
    mQueryModel->setQuery(mQueryModel->query().lastQuery());
    //    QSqlQueryModel * model = qobject_cast<QSqlQueryModel*> (mTreeView->model());
    // model->setQuery(mFilterModel);
}
void AbstractTableWidget::filterChanged(const QString& filter)
{
    if ( mFilterModel!=0)
    {
        mFilterModel->setFilterRegExp(QRegExp(filter, Qt::CaseInsensitive));
        mFilterModel->setFilterKeyColumn(mComboField->currentIndex());
    }
}
QSqlRecord AbstractTableWidget::firstRecordSelected()
{
    if ( mFilterModel!=0)
    {
        if (mTreeView->selectionModel()->selectedRows().count()>0)
        {
            QModelIndex index = mTreeView->selectionModel()->selectedRows().at(0);
            int row  =mFilterModel->mapToSource(index).row();
            return mQueryModel->record(row);
        }
    }
    return QSqlRecord();
}
QList<QSqlRecord> AbstractTableWidget::recordsSelected()
{
    if (mTreeView->selectionModel()->selectedRows().count()>0)
    {
        QList <QSqlRecord> records;
        QModelIndexList indexes = mTreeView->selectionModel()->selectedRows();
        foreach ( QModelIndex index, indexes)
        {
            int row  =mFilterModel->mapToSource(index).row();
            records.append(mQueryModel->record(row));
        }
        return records;
    }
}

void AbstractTableWidget::itemSelected(const QModelIndex &index)
{

    mTreeView->clearSelection();

}
void AbstractTableWidget::exportCsv()
{
    qDebug()<<"export";
    QString fileName = QFileDialog::getSaveFileName(this,"export csv", "", "Csv Files (*.csv)");
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out(&file);
    QSqlRecord record = mQueryModel->record(0);

    for ( int i=0; i<mQueryModel->rowCount();++i)
    {


        for ( int j=0; j<record.count(); ++j)
        {
            QString value =mQueryModel->record(i).value(j).toString();
            out<<value<<",";

        }
        out<<"\n";
    }
    file.close();

    QDesktopServices::openUrl(QUrl(fileName));
}
